/*! \file

\verbatim

Copyright (c) 2004, Sylvain Paris and Francois Sillion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    
    * Redistributions in binary form must reproduce the above
    copyright notice, this list of conditions and the following
    disclaimer in the documentation and/or other materials provided
    with the distribution.

    * Neither the name of ARTIS, GRAVIR-IMAG nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

\endverbatim
*  This file defines the abstract class that is the base of all the other functions.
*/
  
\endverbatim

#ifndef __BASE_2D__
#define __BASE_2D__


namespace Function_2D{


//! Base class for the other functions.
/*!  Must be derived to define the  base_value_proxy() function. One should
  consider defining a static base_value() and to code as following:
  \code

  // virtual inline
  Gaussian::real_type Gaussian::base_value_proxy(const real_type x,
						 const real_type y) const{
    return Gaussian::base_value(x,y);
  }

  // static inline
  Gaussian::real_type Gaussian::base_value(const real_type x,
					   const real_type y){
    return exp(-(x*x + y*y));
  }

  \endcode
  It helps avoiding virtual multiple inheritance by providing a static function.
*/
  class Base_2D{

  public:
    typedef double real_type;

    inline Base_2D(const real_type in_offset_x = 0,
		   const real_type in_offset_y = 0,
		   const real_type in_scale_x = 1,
		   const real_type in_scale_y = 1,
		   const real_type in_rotation_angle = 0,
		   const real_type out_offset = 0,
		   const real_type out_scale = 1);
  
    virtual ~Base_2D();

    inline real_type operator()(const real_type x,
				const real_type y) const;
    
  
  protected:
    virtual inline real_type base_value_proxy(const real_type x,
					      const real_type y) const=0;

    inline void update_input_rotation(const real_type a);
    
  
    //@{
    //! This value is added to the corresponding input coordinate before rotation
    //! and scaling.
  
    real_type input_offset_x,input_offset_y;
    //@}

    //@{
    //! This value is used to scale the corresponding input coordinate
    //! after rotation and offset.
  
    real_type input_scale_x,input_scale_y;
    //@}

    //! The input coordinates are rotated by this angle after being translated
    //! and before being scaled.
    real_type input_rotation_angle;

    //@{
    //! (Co)sine of the input rotation angle.
  
    real_type cos_a,sin_a;
    //@}

    //! The output value is scaled by this value before being translated.
    real_type output_scale;

    //! The output value is translated by this value after scaling.
    real_type output_offset;
  

  };

  

//! Provides a direct interface to the class Base_2D.
  class Direct_interface_2D:public Base_2D{

  public:

    inline Direct_interface_2D(const real_type in_offset_x = 0,
			       const real_type in_offset_y = 0,
			       const real_type in_scale_x = 1,
			       const real_type in_scale_y = 1,
			       const real_type in_rotation_angle = 0,
			       const real_type out_offset = 0,
			       const real_type out_scale = 1);
    
    inline void set_input_offset(const real_type x,const real_type y);
    inline void set_input_scale(const real_type x,const real_type y);
    inline void set_input_rotation(const real_type a);

    inline void set_output_offset(const real_type o);
    inline void set_output_scale(const real_type s);

  };




  //! Extends Base_2D class to control scale with sigma values.
  /*! If the base function is \f$ f_b \f$, the function is controlled through
    \f$ \sigma^{-1}f_b(\frac{x}{\sigma}) \f$. The filter is also controled
    through the center (\f$ center = -offset \f$) and orientation
    (\f$ orientation=-rotation \f$).*/
  class Filter_interface_2D:public Base_2D{

  public:

    inline Filter_interface_2D(const real_type center_x = 0,
			       const real_type center_y = 0,
			       const real_type sigma_x = 1,
			       const real_type sigma_y = 1,
			       const real_type sig_scale_x = 1,
			       const real_type sig_scale_y = 1,
			       const real_type orientation_angle = 0,
			       const real_type out_sigma_scale = 1);


    inline void set_center(const real_type x,const real_type y);
    inline void set_orientation(const real_type o);
    inline void set_sigma(const real_type x,const real_type y);

  protected:
    //@{
    //! Coordinates are scaled by \f$ \frac{\sigma_{scale}}{\sigma} \f$.
    
    real_type sigma_scale_x,sigma_scale_y;
    //@}

    //! Output is scaled by \f$ \frac{out_sigma_scale}{\sigma_x\sigma_y} \f$.
    real_type output_sigma_scale;
};




/*
  ###########################################################
  ###########################################################
  ###########################################################
  ##############                               ##############
  ##############  I M P L E M E N T A T I O N  ##############
  ##############                               ##############
  ###########################################################
  ###########################################################
  ###########################################################
*/


  /*

  ###########
  # Base_2D #
  ###########

  */

  Base_2D::Base_2D(const real_type in_offset_x,
		   const real_type in_offset_y,
		   const real_type in_scale_x,
		   const real_type in_scale_y,
		   const real_type in_rotation_angle,
		   const real_type out_offset,
		   const real_type out_scale):
  
    input_offset_x(in_offset_x),
    input_offset_y(in_offset_y),
    input_scale_x(in_scale_x),
    input_scale_y(in_scale_y),
    input_rotation_angle(in_rotation_angle),
    cos_a(cos(in_rotation_angle)),
    sin_a(sin(in_rotation_angle)),
    output_scale(out_scale),
    output_offset(out_offset){
  }

  Base_2D::~Base_2D(){
  }

  Base_2D::real_type Base_2D::operator()(const real_type x,
					 const real_type y) const{


    const real_type off_x = x + input_offset_x;
    const real_type off_y = y + input_offset_y;
    
    const real_type input_x = (off_x*cos_a + off_y*sin_a)*input_scale_x;
    const real_type input_y =(-off_x*sin_a + off_y*cos_a)*input_scale_y;
    
    return base_value_proxy(input_x,input_y)*output_scale + output_offset;
  }

  void Base_2D::update_input_rotation(const real_type a){
    input_rotation_angle = a;
    cos_a = cos(input_rotation_angle);
    sin_a = sin(input_rotation_angle);
  }


  
/*

  #######################
  # Direct_interface_2D #
  #######################

*/

  Direct_interface_2D::Direct_interface_2D(const real_type in_offset_x,
					   const real_type in_offset_y,
					   const real_type in_scale_x,
					   const real_type in_scale_y,
					   const real_type in_rotation_angle,
					   const real_type out_offset,
					   const real_type out_scale):
    Base_2D(in_offset_x,in_offset_y,
	    in_scale_x,in_scale_y,
	    in_rotation_angle,
	    out_offset,out_scale){}


  
  void Direct_interface_2D::set_input_offset(const real_type x,
					     const real_type y){

    input_offset_x = x;
    input_offset_y = y;
  }

  void Direct_interface_2D::set_input_scale(const real_type x,
					    const real_type y){
  
    input_scale_x = x;
    input_scale_y = y;
  }

  void Direct_interface_2D::set_input_rotation(const real_type a){
  
    update_input_rotation(a);
  }

  void Direct_interface_2D::set_output_scale(const real_type s){

    output_scale = s;
  }
  
  void Direct_interface_2D::set_output_offset(const real_type o){

    output_offset = o;
  }


  
/*

  #######################
  # Filter_interface_2D #
  #######################

*/


  Filter_interface_2D::Filter_interface_2D(const real_type center_x,
					   const real_type center_y,
					   const real_type sigma_x,
					   const real_type sigma_y,
					   const real_type sig_scale_x,
					   const real_type sig_scale_y,
					   const real_type orientation_angle,
					   const real_type out_sigma_scale):
    Base_2D(-center_x,-center_y,
	    sig_scale_x/sigma_x,
	    sig_scale_y/sigma_y,
	    -orientation_angle,
	    0,
	    out_sigma_scale/(sigma_x*sigma_y)),
  sigma_scale_x(sig_scale_x),
  sigma_scale_y(sig_scale_y),
  output_sigma_scale(out_sigma_scale){
  }


  void Filter_interface_2D::set_center(const real_type x,const real_type y){
    
    input_offset_x = -x;
    input_offset_y = -y;
  }
  
  void Filter_interface_2D::set_orientation(const real_type o){
    
    update_input_rotation(-o);
  }

  void Filter_interface_2D::set_sigma(const real_type x,const real_type y){

    input_scale_x = sigma_scale_x / x;
    input_scale_y = sigma_scale_y / y;
    output_scale  = output_sigma_scale / (x*y);
  }


}
  
#endif
